#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# Oct 2015  Michael     Initial Authoring

import os,sys,inspect,time,thread
import socket,fcntl,struct,ms_explorerlib,subprocess    

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
from PiStorms import PiStorms

print "running program"
psm = PiStorms()
psm.screen.termPrintln("Connect your I2C device to BAS1")
psm.screen.termPrintln("and click Explore")
psm.screen.drawButton(75, 95, width = 85, height = 40, text="Explore", display=False)
psm.screen.drawButton(175, 95, width = 60, height = 40, text="Exit", display=True)

exit = False
lastled = 0

def explore():
    psm.screen.termPrintAt(5, "Searching for i2c device...")        
    psm.BAS1.activateCustomSensorI2C()
    time.sleep(3)
    addr = 0x00     # DO NOT change this address!!!
    i2c = ms_explorerlib.Explorer(addr)
    found = i2c.ping(0x00)  
    # Checks for connection on all I2C addresses until connection is found
    count = 0 
    while found == -1:     
        if (addr < 0xef):
            addr = addr + 1
            count = count + 1
            if (addr != 0x34 and addr != 0x35 and addr != 0x36 and addr != 0x37):
                i2c = ms_explorerlib.Explorer(addr)
                found = i2c.ping(0x00)
            if (count > 2000):
                found = 5
        else:
            addr = 0x00         
    if (found == 5):
        psm.screen.termPrintAt(5, "No Device found!")
        psm.screen.termPrintAt(6, "Click Exit to return to main menu")
        time.sleep(2)
        global exit
        exit = True
    else:
        global currAddr
        currAddr = addr
        #psm.screen.termPrintAt(5, "8 bit address: " + str(hex(addr)))      
        
def selectAddress():
    psm.screen.clearScreen()
    psm.screen.drawDisplay("PiStorms")
    psm.screen.drawArrows()
    psm.screen.termPrintAt(0, "Use arrows to choose i2c address")
    psm.screen.termPrintAt(1, "and click Change")
    psm.screen.drawButton(75, 95, width = 85, height = 40, text="Change", display=False)
    psm.screen.drawButton(175, 95, width = 60, height = 40, text="Exit", display=True)
    psm.screen.termPrintAt(5, "Address: " + str(hex(currAddr)))
    global nextAddr
    nextAddr = currAddr + 2
    psm.screen.termPrintAt(6, "New Address: ")
    psm.screen.termPrintAt(7, "       " + str(hex(nextAddr)))
    check = psm.screen.checkButton(75, 95,width=85,height=40)
    while(check == False): 
        check = psm.screen.checkButton(75, 95,width=85,height=40)
        bye = psm.screen.checkButton(175, 95,width=60,height=40)         
        if(psm.screen.checkArrows() == (False, True)):
            nextAddr = nextAddr + 2
        if(psm.screen.checkArrows() == (True, False)):
            nextAddr = nextAddr - 2        
        if(bye == True):
            psm.screen.termPrintAt(9, "Exiting to menu")
            global exit
            exit = True   
            check = True            
        psm.screen.termPrintAt(7, "       " + str(hex(nextAddr)))        
        #time.sleep(.25)
    
def changeAddress():
    psm.screen.termPrintAt(8, "Changing Address...")
    command = "/home/pi/PiStormsprograms/addresschange " + str(hex(currAddr)) + " " + str(hex(nextAddr))
    psm.screen.termPrintAt(9, command)
    #os.system
    subprocess.call(command, shell=True)  
    time.sleep(2)
    global exit
    exit = True
    
while(not exit):
    action = psm.screen.checkButton(75, 95,width=85,height=40)
    bye = psm.screen.checkButton(175, 95,width=60,height=40)
    #currAddr = 356
    #nextAddr = 36
    #command = './addresschange ' + str(hex(currAddr)) + " " + str(hex(nextAddr))
    #subprocess.call(command, shell=True) 
    if(action == True): 
        explore()
        if(exit == False):
            selectAddress()
            time.sleep(.25)
            if(exit == False):
                changeAddress()            
    if(bye == True):
        psm.screen.termPrintAt(9, "Exiting to menu")
        exit = True
    time.sleep(.1)


#!/usr/bin/env python
#
# Copyright (c) 2014 OpenElectrons.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# History:
# Date       Author           Comments
# 07/26/14   Michael Giles   Initial authoring.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
#
# Default host = '0.0.0.0' or host = 'localhost' for use on Raspberry Pi
# To connect Raspberry Pi to PC set host equal to the computer's IP address
# eg. host = '127.0.0.1'
#host = '10.21.5.152'
host = '0.0.0.0'
# Default Port 42001
port = 42001


#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/

# History:
# Date      Author      Comments
#  July 2015  Henry     Initial Authoring 

from mindsensors_i2c import mindsensors_i2c
import time, math
import sys,os
import ctypes
import random

class PSSensor():

    PS_SENSOR_TYPE_NONE = 0
    PS_SENSOR_TYPE_SWITCH = 1
    PS_SENSOR_TYPE_ANALOG = 2
    PS_SENSOR_TYPE_LIGHT_ACTIVE = 3
    PS_SENSOR_TYPE_LIGHT_INACTIVE = 4
    PS_SENSOR_TYPE_SOUND_DB = 5
    PS_SENSOR_TYPE_SOUND_DBA = 6
    PS_SENSOR_TYPE_LOWSPEED_9V = 7
    PS_SENSOR_TYPE_LOWSPEED = 8
    PS_SENSOR_TYPE_CUSTOM = 9
    PS_SENSOR_TYPE_COLORFULL = 13
    PS_SENSOR_TYPE_COLORRED = 14
    PS_SENSOR_TYPE_COLORGREEN = 15
    PS_SENSOR_TYPE_COLORBLUE = 16
    PS_SENSOR_TYPE_COLORNONE = 17
    PS_SENSOR_TYPE_EV3_SWITCH = 18
    PS_SENSOR_TYPE_EV3 = 19
    
    
    PS_EV3CACHE_READY = 0
    PS_EV3CACHE_ID = 1
    PS_EV3CACHE_READY = 2
    PS_EV3CACHE_READY = 3
    PS_EV3CACHE_READY = 4
    
    
    
    sensornum = 0
    def __init__(self,bank,num):
        self.bank = bank
        self.sensornum = num
        self.type = self.PS_SENSOR_TYPE_NONE
        self.EV3Cache = [0,[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],0,0,[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]]
    def setType(self,type):
        if(type != self.type):
            self.type = type
            if(self.sensornum == 1):
                self.bank.writeByte(PiStormsCom.PS_S1_Mode,type)
            if(self.sensornum == 2):
                self.bank.writeByte(PiStormsCom.PS_S2_Mode,type)
            if(self.type != self.PS_SENSOR_TYPE_CUSTOM):
                time.sleep(1)
    def getType(self):
        return self.type
    def EV3Retrieve(self):
        if(self.sensornum == 1):
            self.EV3Cache[0] = self.bank.readByte(PiStormsCom.PS_S1EV_Ready)
            self.EV3Cache[1] = self.bank.readArray(PiStormsCom.PS_S1EV_SensorID,16)
            self.EV3Cache[2] = self.bank.readByte(PiStormsCom.PS_S1EV_Mode)
            self.EV3Cache[3] = self.bank.readByte(PiStormsCom.PS_S1EV_Length)
            self.EV3Cache[4] = self.bank.readArray(PiStormsCom.PS_S1EV_Data,32)
        if(self.sensornum == 2):
            self.EV3Cache[0] = self.bank.readByte(PiStormsCom.PS_S2EV_Ready)
            self.EV3Cache[1] = self.bank.readArray(PiStormsCom.PS_S2EV_SensorID,16)
            self.EV3Cache[2] = self.bank.readByte(PiStormsCom.PS_S2EV_Mode)
            self.EV3Cache[3] = self.bank.readByte(PiStormsCom.PS_S2EV_Length)
            self.EV3Cache[4] = self.bank.readArray(PiStormsCom.PS_S2EV_Data,32)
    def isTouchedEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3_SWITCH)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0] == 1
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data) == 1
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data) == 1     
    def numTouchesEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3_SWITCH)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][1]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data+1)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data+1)
    def resetTouchesEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3_SWITCH)
        if(self.sensornum == 1):
            self.bank.writeByte(PiStormsCom.PS_S1EV_Data + 1,0)
        if(self.sensornum == 2):
            self.bank.writeByte(PiStormsCom.PS_S2EV_Data + 1,0)
    def setModeEV3(self, mode):
        if(self.sensornum == 1):
            self.bank.writeByte(PiStormsCom.PS_S1EV_Mode,mode)
        if(self.sensornum == 2):
            self.bank.writeByte(PiStormsCom.PS_S2EV_Mode,mode)
    def distanceIREV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #raw1 = self.EV3Cache[4][0]
        #raw2 = self.EV3Cache[4][1]
        #return ctypes.c_short(raw1 | (raw2*256)).value
        if(self.sensornum == 1):
            return self.bank.readInteger(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readInteger(PiStormsCom.PS_S2EV_Data)
    def rawIREV3(self,mode):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(mode)
        #self.EV3Retrieve()
        #return self.EV3Cache[4]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data) 
    def headingIREV3(self,channel):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(1)
        #return ctypes.c_byte(self.rawIREV3(1)[(channel-1)*2]).value
        if(self.sensornum == 1):
            return self.bank.readByteSigned(PiStormsCom.PS_S1EV_Data + ((channel-1)*2))
        if(self.sensornum == 2):
            return self.bank.readByteSigned(PiStormsCom.PS_S2EV_Data + ((channel-1)*2))
    def distanceRemoteIREV3(self,channel):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(1)
        #return ctypes.c_byte(self.rawIREV3(1)[((channel-1)*2)+1]).value
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data + (((channel-1)*2)+1))
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data + (((channel-1)*2)+1))
    def remoteLeft(self,channel):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(2)
        #remote = self.rawIREV3(2)[channel-1]
        if(self.sensornum == 1):
            remote = self.bank.readByte(PiStormsCom.PS_S1EV_Data + (channel-1))
        if(self.sensornum == 2):
            remote = self.bank.readByte(PiStormsCom.PS_S2EV_Data + (channel-1))
        if(remote == 0 or remote == 3 or remote == 4):
            return 0
        if(remote == 1 or remote == 5 or remote == 6):
            return 1
        if(remote == 2 or remote == 7 or remote == 8):
            return -1
    def remoteRight(self,channel):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(2)
        #remote = self.rawIREV3(2)[channel-1]
        if(self.sensornum == 1):
            remote = self.bank.readByte(PiStormsCom.PS_S1EV_Data + (channel-1))
        if(self.sensornum == 2):
            remote = self.bank.readByte(PiStormsCom.PS_S2EV_Data + (channel-1))
        if(remote == 0 or remote == 1 or remote == 2):
            return 0
        if(remote == 3 or remote == 7 or remote == 5):
            return 1
        if(remote == 4 or remote == 6 or remote == 8):
            return -1
    def distanceUSEV3cm(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #raw1 = self.EV3Cache[4][0]
        #raw2 = self.EV3Cache[4][1]
        #return ctypes.c_short(raw1 | (raw2*256)).value
        if(self.sensornum == 1):
            return self.bank.readInteger(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readInteger(PiStormsCom.PS_S2EV_Data)
    def distanceUSEV3in(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(1)
        #self.EV3Retrieve()
        #raw1 = self.EV3Cache[4][0]
        #raw2 = self.EV3Cache[4][1]
        #return ctypes.c_short(raw1 | (raw2*256)).value
        if(self.sensornum == 1):
            return self.bank.readInteger(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readInteger(PiStormsCom.PS_S2EV_Data)
    def presenceUSEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(2)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0] == 1
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data) == 1
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data) == 1
    def rawGyro(self, mode):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(mode)
        #self.EV3Retrieve()
        #return self.EV3Cache[4]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data)
    def gyroAngleEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(0)
        #raw = self.rawGyro(0)
        #raw1 = raw[0]
        #raw2 = raw[1]
        #return ctypes.c_short(raw1 | (raw2*256)).value
        if(self.sensornum == 1):
            return self.bank.readIntegerSigned(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readIntegerSigned(PiStormsCom.PS_S2EV_Data)
    def gyroRateEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(1)
        #raw = self.rawGyro(1)
        #raw1 = raw[0]
        #raw2 = raw[1]
        #return ctypes.c_short(raw1 | (raw2*256)).value
        if(self.sensornum == 1):
            return self.bank.readIntegerSigned(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readIntegerSigned(PiStormsCom.PS_S2EV_Data)
    def reflectedLightSensorEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data)
    def ambientLightSensorEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(1)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data)
    PS_SENSOR_COLOR_NONE = 0
    PS_SENSOR_COLOR_BLACK = 1
    PS_SENSOR_COLOR_BLUE = 2
    PS_SENSOR_COLOR_GREEN = 3
    PS_SENSOR_COLOR_YELLOW = 4
    PS_SENSOR_COLOR_RED = 5
    PS_SENSOR_COLOR_WHITE = 6
    PS_SENSOR_COLOR_BROWN = 7
    def colorSensorEV3(self):
        self.setType(self.PS_SENSOR_TYPE_EV3)
        self.setModeEV3(2)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data)
    def readNXT(self):
        if(self.sensornum == 1):
            return self.bank.readInt(PiStormsCom.PS_S1EV_Ready)
        if(self.sensornum == 2):
            return self.bank.readInt(PiStormsCom.PS_S2EV_Ready)
    def isTouchedNXT(self):
        self.setType(self.PS_SENSOR_TYPE_SWITCH)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][0] == 1
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data) == 1
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data) == 1
    def numTouchesNXT(self):
        self.setType(self.PS_SENSOR_TYPE_SWITCH)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return self.EV3Cache[4][1]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Data + 1)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Data + 1)
    def resetTouchesNXT(self):
        if(self.sensornum == 1):
            self.bank.writeByte(PiStormsCom.PS_S1EV_Data+1,0)
        if(self.sensornum == 2):
            self.bank.writeByte(PiStormsCom.PS_S2EV_Data+1,0)
    def lightSensorNXT(self, active=True):
        if(active):
            self.setType(self.PS_SENSOR_TYPE_LIGHT_ACTIVE)
        else:
            self.setType(self.PS_SENSOR_TYPE_LIGHT_INACTIVE)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return (self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]
        if(self.sensornum == 1):
            return self.bank.readInteger(PiStormsCom.PS_S1EV_Ready)
        if(self.sensornum == 2):
            return self.bank.readInteger(PiStormsCom.PS_S2EV_Ready)
        
    def SumoEyes(self, long = True):
        self.SE_None = 0
        self.SE_Front = 1
        self.SE_Left = 2
        self.SE_Right = 3
        if(long):
            self.setType(self.PS_SENSOR_TYPE_LIGHT_INACTIVE)
        else:
            self.setType(self.PS_SENSOR_TYPE_LIGHT_ACTIVE) 
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #if  self.SumoEyesisNear(465, 30,(self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]   ):
        if(self.sensornum == 1):
            if(self.SumoEyesisNear(465, 30, (self.bank.readInteger(PiStormsCom.PS_S1EV_Ready)))):
                return self.SE_Front
        #if  self.SumoEyesisNear(800, 30, (self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]  ):
            elif(self.SumoEyesisNear(800, 30, (self.bank.readInteger(PiStormsCom.PS_S1EV_Ready)))):
                return self.SE_Left
        #if  self.SumoEyesisNear(555, 30, (self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]  ):
            elif(self.SumoEyesisNear(800, 30, (self.bank.readInteger(PiStormsCom.PS_S1EV_Ready)))):
                return self.SE_Right
            else:
                return self.SE_None 
        if(self.sensornum == 2):
            if(self.SumoEyesisNear(465, 30, (self.bank.readInteger(PiStormsCom.PS_S2EV_Ready)))):
                return self.SE_Front
        #if  self.SumoEyesisNear(800, 30, (self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]  ):
            elif(self.SumoEyesisNear(800, 30, (self.bank.readInteger(PiStormsCom.PS_S2EV_Ready)))):
                return self.SE_Left
        #if  self.SumoEyesisNear(555, 30, (self.EV3Cache[1][0]<<8 ) +self.EV3Cache[0]  ):
            elif(self.SumoEyesisNear(800, 30, (self.bank.readInteger(PiStormsCom.PS_S2EV_Ready)))):
                return self.SE_Right
            else:
                return self.SE_None                  
        
    def SumoEyesisNear(self,reference, delta, comet):
        if (comet > (reference - delta)) and (comet < (reference + delta)):
            return True
        else:
            return False        
        
    def colorSensorRawNXT(self, smode = 13):
        self.setType(smode)
        self.setModeEV3(0)
        self.EV3Retrieve()
        return self.EV3Cache[0:1] + self.EV3Cache[1]
    def colorSensorNXT(self, smode = 13):
        self.setType(smode)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return self.EV3Cache[0]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Ready)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Ready)
    def colorSensorNoneNXT(self):
        return self.colorSensorRawNXT(self.PS_SENSOR_TYPE_COLORNONE)[0]
    def colorSensorRedNXT(self):
        return self.colorSensorRawNXT(self.PS_SENSOR_TYPE_COLORRED)[0]
    def colorSensorGreenNXT(self):
        return self.colorSensorRawNXT(self.PS_SENSOR_TYPE_COLORGREEN)[0]
    def colorSensorBlueNXT(self):
        return self.colorSensorRawNXT(self.PS_SENSOR_TYPE_COLORBLUE)[0]
    def analogSensor(self): #untested
        self.setType(self.PS_SENSOR_TYPE_ANALOG)
        self.setModeEV3(0)
        #self.EV3Retrieve()
        #return self.EV3Cache[0]
        if(self.sensornum == 1):
            return self.bank.readByte(PiStormsCom.PS_S1EV_Ready)
        if(self.sensornum == 2):
            return self.bank.readByte(PiStormsCom.PS_S2EV_Ready)
    def activateCustomSensorI2C(self):
        self.setType(self.PS_SENSOR_TYPE_CUSTOM)
        
    
    
class PSMotor():
    
    motornum = 0
    def __init__(self, bank, num):
        self.bank = bank
        self.motornum = num
    
    def pos(self):
        if(self.motornum == 1):
            return self.bank.readLongSigned(PiStormsCom.PS_Position_M1)
            
        elif(self.motornum == 2):
            return self.bank.readLongSigned(PiStormsCom.PS_Position_M2)
            
    #def resPos(self, bnk):
    #    PiStormsCom.command(82, bnk) 
         
    def setSpeed( self, speed):
        if(speed == 0):
            self.float()
            return
        
        ctrl = 0
        ctrl |= PiStormsCom.PS_CONTROL_SPEED
        
        #print speed
        speed = int(speed)
        
        
        ctrl |= PiStormsCom.PS_CONTROL_GO
        if (self.motornum == 1): 
            array = [speed, 0, 0, ctrl]
            self.bank.writeArray( PiStormsCom.PS_Speed_M1, array)
        if (self.motornum == 2):
            array = [speed, 0, 0, ctrl]
            self.bank.writeArray( PiStormsCom.PS_Speed_M2, array)
    def brake(self):
        if(self.motornum == 1):
            self.bank.writeByte(PiStormsCom.PS_Command, PiStormsCom.A)
        if(self.motornum == 2):
            self.bank.writeByte(PiStormsCom.PS_Command, PiStormsCom.B)
    def float(self):
        if(self.motornum == 1):
            self.bank.writeByte(PiStormsCom.PS_Command, PiStormsCom.a)
        if(self.motornum == 2):
            self.bank.writeByte(PiStormsCom.PS_Command, PiStormsCom.b)
    def hold(self):
        ctrl = 0
        ctrl |= PiStormsCom.PS_CONTROL_BRK
        ctrl |= PiStormsCom.PS_CONTROL_ON
        #ctrl |= PiStormsCom.PS_CONTROL_RELATIVE
        ctrl |= PiStormsCom.PS_CONTROL_TACHO
        ctrl |= PiStormsCom.PS_CONTROL_GO
        
        if(self.motornum == 1):
            self.bank.writeArray(PiStormsCom.PS_SetPoint_M1, [0,0,0,0])
            self.bank.writeByte(PiStormsCom.PS_CMDA_M1, ctrl)
        else:
            self.bank.writeArray(PiStormsCom.PS_SetPoint_M2, [0,0,0,0])
            self.bank.writeByte(PiStormsCom.PS_CMDA_M2, ctrl)
    def runSecs(self,secs,speed, brakeOnCompletion = False):
        ctrl = 0
        ctrl |= PiStormsCom.PS_CONTROL_SPEED
        ctrl |= PiStormsCom.PS_CONTROL_TIME
        if(brakeOnCompletion):
            ctrl |= PiStormsCom.PS_CONTROL_BRK
        ctrl |= PiStormsCom.PS_CONTROL_GO
        if(self.motornum == 1):
            array = [speed,secs,0,ctrl]
            self.bank.writeArray(PiStormsCom.PS_Speed_M1,array)
        if(self.motornum == 2):
            array = [speed,secs,0,ctrl]
            self.bank.writeArray(PiStormsCom.PS_Speed_M2,array)
    def status(self):
        if(self.motornum == 1):
            return self.bank.readByte(PiStormsCom.PS_Status_M1)
        if(self.motornum == 2):
            return self.bank.readByte(PiStormsCom.PS_Status_M2)
    def statusBit(self, bitno = 0):
        return (self.status() >> bitno) & 1
    def isBusy(self):
        return self.statusBit(0) == 1 or self.statusBit(1) == 1 or self.statusBit(3) == 1 or self.statusBit(6) == 1
    def waitUntilNotBusy(self, timeout=-1):
        while(self.isBusy()):
            time.sleep(.01)
            timeout -= 1
            if(timeout == 0):
                return 1
            if(timeout <-5):
                timeout = -1
            pass
        return 0
    def isStalled(self):
        return self.statusBit(7) == 1
    def isOverloaded(self):
        return self.statusBit(5) == 1
    def runDegs(self,degs,speed = 100,brakeOnCompletion = False, holdOnCompletion = False):
        ctrl = 0
        ctrl |= PiStormsCom.PS_CONTROL_SPEED
        ctrl |= PiStormsCom.PS_CONTROL_TACHO
        ctrl |= PiStormsCom.PS_CONTROL_RELATIVE
        if(brakeOnCompletion):
            ctrl |= PiStormsCom.PS_CONTROL_BRK
        if(holdOnCompletion):
            ctrl |= PiStormsCom.PS_CONTROL_BRK
            ctrl |= PiStormsCom.PS_CONTROL_ON
        ctrl |= PiStormsCom.PS_CONTROL_GO
        
        b4 = (degs/0x1000000)
        b3 = ((degs%0x1000000)/0x10000)
        b2 = (((degs%0x1000000)%0x10000)/0x100)
        b1 = (((degs%0x1000000)%0x10000)%0x100)
        
        # b1 = degs & 0xFF
        # b2 = (degs >>8) & 0xFF
        # b3 = (degs >>16) & 0xFF
        # b4 = (degs >> 24) & 0xFF
        
        if(self.motornum == 1):
            array = [b1, b2, b3, b4, speed, 0, 0, ctrl]
            self.bank.writeArray(PiStormsCom.PS_SetPoint_M1, array) 
        if(self.motornum == 2):
            array = [b1, b2, b3, b4, speed, 0, 0, ctrl]
            self.bank.writeArray(PiStormsCom.PS_SetPoint_M2, array) 
    def SetPerformanceParameters(self, Kp_tacho, Ki_tacho, Kd_tacho, Kp_speed, Ki_speed, Kd_speed, passcount, tolerance):#untested
        Kp_t1 = Kp_tacho%0x100
        Kp_t2 = Kp_tacho/0x100    
        Ki_t1 = Ki_tacho%0x100
        Ki_t2 = Ki_tacho/0x100
        Kd_t1 = Kd_tacho%0x100
        Kd_t2 = Kd_tacho/0x100
        Kp_s1 = Kp_speed%0x100      
        Kp_s2 = Kp_speed/0x100
        Ki_s1 = Ki_speed%0x100 
        Ki_s2 = Ki_speed/0x100
        Kd_s1 = Kd_speed%0x100
        Kd_s2 = Kd_speed/0x100
        passcount = passcount
        tolerance = tolerance
        array = [Kp_t1 , Kp_t2 , Ki_t1, Ki_t2, Kd_t1, Kd_t2, Kp_s1, Kp_s2, Ki_s1, Ki_s2, Kd_s1, Kd_s2, passcount, tolerance]
        self.bank.writeArray(self.PS_P_Kp, array)       
    
class PiStormsCom():

    PS_A_ADDRESS = 0x34
    PS_B_ADDRESS = 0x36
    
    # Registers
    PS_Command = 0x41
    
    PS_SetPoint_M1 =0x42
    PS_Speed_M1 = 0x46
    PS_Time_M1 = 0x47
    PS_CMDB_M1 = 0x48
    PS_CMDA_M1 = 0x49
    
    PS_SetPoint_M2 =0x4A
    PS_Speed_M2 = 0x4E
    PS_Time_M2 = 0x4F
    PS_CMDB_M2 = 0x50
    PS_CMDA_M2 = 0x51
    
    PS_Position_M1 = 0x52
    PS_Position_M2 = 0x56
    
    PS_Status_M1 = 0x5A
    PS_Status_M2 = 0x5B
    
    PS_Tasks_M1 = 0x5C
    PS_Tasks_M2 = 0x5D
    
    PS_P_Kp = 0x5E
    PS_P_Ki = 0x60
    PS_P_Kd = 0x62
    PS_S_Kp = 0x64
    PS_S_Ki = 0x66
    PS_S_Kd = 0x68
    PS_PassCount = 0x6A
    PS_PassTolerance = 0x6B
    PS_ChkSum = 0x6C
    PS_BattV = 0x6E
    # Sensor 1
    PS_S1_Mode = 0x6F
    # EV3
    PS_S1EV_Ready = 0x70
    PS_S1EV_SensorID = 0x71
    PS_S1EV_Mode = 0x81
    PS_S1EV_Length = 0x82
    PS_S1EV_Data = 0x83
    # LEGO Analog
    PS_S1AN_Read = 0x70
    # LEGO Color
    PS_S1C_Color = 0x70
    PS_S1C_R_cal = 0x71
    PS_S1C_G_cal = 0x72
    PS_S1C_B_cal = 0x73
    PS_S1C_N_cal = 0x74
    PS_S1C_R_raw = 0x75
    PS_S1C_G_raw = 0x76
    PS_S1C_B_raw = 0x77
    PS_S1C_N_raw = 0x78
    # Sensor 2
    PS_S2_Mode = 0xA3
    # EV3
    PS_S2EV_Ready = 0xA4
    PS_S2EV_SensorID = 0xA5
    PS_S2EV_Mode = 0xB5
    PS_S2EV_Length = 0xB6
    PS_S2EV_Data = 0xB7
    # LEGO Analog
    PS_S2AN_Read = 0xA4
    # LEGO Color
    PS_S2C_Color = 0xA4
    PS_S2C_R_cal = 0xA5
    PS_S2C_G_cal = 0xA6
    PS_S2C_B_cal = 0xA7
    PS_S2C_N_cal = 0xA8
    PS_S2C_R_raw = 0xA9
    PS_S2C_G_raw = 0xAA
    PS_S2C_B_raw = 0xAB
    PS_S2C_N_raw = 0xAC
    # LED 
    PS_R = 0xD7
    PS_G = 0xD8
    PS_B = 0xD9
    # Buttons
    PS_KeyPress = 0xDA
    PS_Key1Count = 0xDB
    PS_Key2Count = 0xDC
    
    PS_CONTROL_SPEED  = 0x01
    PS_CONTROL_RAMP   = 0x02
    PS_CONTROL_RELATIVE =  0x04
    PS_CONTROL_TACHO  = 0x08
    PS_CONTROL_BRK  =   0x10
    PS_CONTROL_ON    =  0x20
    PS_CONTROL_TIME  =   0x40
    PS_CONTROL_GO   =     0x80
    #Supported I2C commands
    R = 0x52
    S = 0x53
    a = 0x61
    b = 0x62
    c = 0x63
    A = 0x41
    B = 0x42
    C = 0x43
    H = 0x48
    
    
    bankA = mindsensors_i2c(PS_A_ADDRESS >> 1)
    bankB = mindsensors_i2c(PS_B_ADDRESS >> 1)
    
    BAM1 = PSMotor(bankA,1)
    BAM2 = PSMotor(bankA,2)
    BBM1 = PSMotor(bankB,1)
    BBM2 = PSMotor(bankB,2)
    
    BAS1 = PSSensor(bankA,1)
    BAS2 = PSSensor(bankA,2)
    BBS1 = PSSensor(bankB,1)
    BBS2 = PSSensor(bankB,2)
    
    def __init__(self):
        try:
            self.bankA.readByte(self.PS_BattV)
        except:
            print "could not connect to pistorms"
        else:
            self.bankA.writeByte(self.PS_Command,self.R)
            self.bankB.writeByte(self.PS_Command,self.R)
            
    def resPosA(self):
        self.bankA.writeByte(self.PS_Command,self.R)
        
    def resPosB(self):
        self.bankB.writeByte(self.PS_Command,self.R)
        
    def Shutdown(self):
        self.bankA.writeByte(self.PS_Command,self.H)
        
    def command(self, cmd, bank):
        if(bank == 1):
            self.bankA.writeByte(self.PS_Command,cmd)
        elif(bank == 2):
            self.bankB.writeByte(self.PS_Command,cmd)
            
    def battVoltage(self):
        return self.bankA.readByte(self.PS_BattV)*.040
    ##  Read the firmware version of the i2c device
    
    def GetFirmwareVersion(self):
        ver = self.bankA.readString(0x00, 8)
        return ver

    ##  Read the vendor name of the i2c device
    def GetVendorName(self):
        vendor = self.bankA.readString(0x08, 8)
        return vendor

    ##  Read the i2c device id
    def GetDeviceId(self):
        device = self.bankA.readString(0x10, 8)
        return device    
        
    def led(self,lednum,red,green,blue):
    
        try:
            if(lednum == 1):
            
                array = [red, green, blue]
                self.bankA.writeArray(self.PS_R, array)
            if(lednum == 2):
                array = [red, green, blue]
                self.bankB.writeArray(self.PS_R, array)
        except AttributeError:
            self.led(lednum,red,green,blue)
        time.sleep(.001)
    def isKeyPressed(self):
        return int(0x01&self.bankA.readByte(self.PS_KeyPress))
    def getKeyPressValue(self):
        return self.bankA.readByte(self.PS_KeyPress)
    def getKeyPressCountx(self):
        return self.bankA.readByte(self.PS_Key1Count)
    def resetKeyPressCount(self):
        self.bankA.writeByte(self.PS_Key1Count,0)
    def ping(self):
        self.bankA.readByte(0x00)
if __name__ == '__main__':
    psc = PiStormsCom()
    print "Version = "+ str(psc.GetFirmwareVersion())
    print "Vendor = "+ str(psc.GetVendorName())
    print "Device = "+ str(psc.GetDeviceId())
    try:
        while(True):
            print   psc.battVoltage()
            print psc.BAS1.SumoEyes(True)
            print psc.BAS2.colorSensorNXT()
            print psc.BBS1.lightSensorNXT(True)
            psc.BAM1.runSecs(1,100,True)
            psc.BBM1.runSecs(1,100,True)
            psc.BAM2.runSecs(1,100,True)
            psc.BBM2.runSecs(1,100,True)
            psc.BAM1.waitUntilNotBusy()
            
            psc.BAM1.runSecs(1,-100,True)
            psc.BBM1.runSecs(1,-100,True)
            psc.BAM2.runSecs(1,-100,True)
            psc.BBM2.runSecs(1,-100,True)
            psc.BAM1.waitUntilNotBusy()
            
            psc.BAM1.hold()
            psc.BAM2.hold()
            psc.BBM1.hold()
            psc.BBM2.hold()
            
            time.sleep(5)
            psc.BAM1.float()
            psc.BAM2.float()
            psc.BBM1.float()
            psc.BBM2.float()
            time.sleep(1)
            
    except KeyboardInterrupt:
        psc.BAM1.float()
        psc.BAM2.float()
        psc.BBM1.float()
        psc.BBM2.float()















#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
#  Spet 2015  Nitin     Initial Authoring from SensorShield import SensorShield





#!/usr/bin/env python
#
# Copyright (c) 2014 OpenElectrons.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date       Author           Comments
# 07/26/14   Michael Giles   Initial authoring.
# 08/18/14   Michael Giles   Servo 
# 10/18/15   Nitin           PiStorm integration
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
import sys,rmap,threading,time,math,socket,scratch,subprocess,datetime
import rmapcfg
from PiStorms import PiStorms

debug = False
device_dict = rmap.dict
read_dict = rmap.read_dict
command_array = []
command_array1 = []
string_message = ""
psm = PiStorms()
lasttime = int(round(time.time() * 1000))
# Connect to Scratch
try:
    s = scratch.Scratch(host=rmapcfg.host, port=rmapcfg.port)
    if s.connected:
        rmap.rmap_print("Connected")
except scratch.ScratchError:
    rmap.rmap_print("Scratch is either not opened or remote sensor connections aren't enabled")

# Broadcast 'READY' to Scratch
try:
    s.broadcast('READY')
    time.sleep(.5)
except NameError:
    #pass
    print "Did not send Ready"

doExit = False
# Handles messages received from Scratch
while doExit == False:
    try:
        # Receives message from Scratch
        msg = s.receive()
        
        
        # Checks for python format documentation
        if msg[0] == 'broadcast':
            command_array1 = []
            string_message = ""
            if ('@' in msg[1] )and ('#' in msg[1]):
                pos = msg[1].find('#')
                pos1 = msg[1].find('@')
                if pos > pos1 :
                    command_array1.append(msg[1][0:pos1])
                    string_message = string_message+(msg[1][pos1+1:pos])
                else :
                    command_array1.append(msg[1][0:pos])
                    string_message = string_message+(msg[1][pos1+1:])                    
                    
                
            elif '#' in msg[1]: 
                pos = msg[1].find('#')
                command_array1.append(msg[1][0:pos])
            elif '@' in msg[1]: 
                pos = msg[1].find('@')
                command_array1.append(msg[1][0:pos])
                string_message = string_message+(msg[1][pos+1:])
            else:    
                command_array1.append(msg[1])
            
        command_array = []        
        for cmd in command_array1:
            command_array.append(cmd.upper())
        if(debug) :
            print  command_array ,string_message,"DT = ",int(round(time.time() * 1000)) -lasttime 
            lasttime = int(round(time.time() * 1000))
        # Split message into words
        for cmd in command_array:  
            split_cmd = cmd.split()
            # Creates and assigns object to a variable and store in device_dict
            #print device_dict
            if (split_cmd[0] == 'CR'):
                if len(split_cmd) > 2:
                    s.sensorupdate({"RMAP Status" : 1})  
                    if (split_cmd[2] in device_dict):
                        pass
                    else:
                        device_dict[split_cmd[2]] = rmap.device_cr(split_cmd[1], split_cmd[2])#, split_cmd[3])
                else:
                    s.sensorupdate({"RMAP Status" : 2})
                    rmap.rmap_print("Error:Command too short, expects 3 received "+str(len(cmd))+":"+cmd)                    
            # Creates variable lists in read_dict
            elif (split_cmd[0] == 'RD'):
                if len(split_cmd) > 2:
                    x = 0
                    max = len(cmd) + 1 
                    if (split_cmd[1] in device_dict):
                        s.sensorupdate({"RMAP Status" : 1})  
                        if (split_cmd[1] in read_dict):
                            pass
                        else:
                            read_dict[split_cmd[1]] = []
                        # Creates variables in variable lists in read_dict
                        ''' for word in split_cmd:
                                if x > 1:
                                    if (str(split_cmd[1])+'_'+str(split_cmd[x]) in read_dict[split_cmd[1]]): 
                                        pass
                                    else:
                                        read_dict[split_cmd[1]].append(str(split_cmd[1])+'_'+str(split_cmd[x]))
                                if x < max:
                                 x = x + 1
                        '''
                        newvar = str(split_cmd[2])
                        for word in split_cmd:
                            if x > 2:
                                newvar =  newvar+': '+str(split_cmd[x])
                            if x < max:
                                x = x + 1 
                        if newvar not in read_dict[split_cmd[1]]: 
                            read_dict[split_cmd[1]].append(newvar)        
                        if(debug) :
                            print read_dict[split_cmd[1]]
                        # Reads data and updates to Scratch
                        '''    if split_cmd[1] in read_dict:
                            if (str(split_cmd[1])+'_'+str(split_cmd[2]) in read_dict[split_cmd[1]]): 
                                listpos = read_dict[split_cmd[1]].index(str(split_cmd[1])+'_'+str(split_cmd[2]))
                                s.sensorupdate({str(read_dict[split_cmd[1]][listpos]) : device_dict[split_cmd[1]].process_read(read_dict[split_cmd[1]][listpos])})                      
                        '''            #time.sleep(.15) 
                        if split_cmd[1] in read_dict:
                            if newvar in read_dict[split_cmd[1]]: 
                                listpos = read_dict[split_cmd[1]].index(newvar)
                                s.sensorupdate({str(read_dict[split_cmd[1]][listpos]) : device_dict[split_cmd[1]].process_read(read_dict[split_cmd[1]][listpos])})                      
                                
                    else:
                        s.sensorupdate({"RMAP Status" : 2}) 
                        rmap.rmap_print("Error:Sensor variable not recognized:"+split_cmd[1])                          
                else:
                    rmap.rmap_print("Error:Command too short, expects 3 received "+str(len(cmd))+":"+cmd)         
                    s.sensorupdate({"RMAP Status" : 2})                      
            # Writes data to sensor
            elif (split_cmd[0] in device_dict): 
                if(debug) :
                    print split_cmd
                if device_dict[split_cmd[0]].run_command(split_cmd, string_message) == True:
                    s.sensorupdate({"RMAP Status" : 1})
                else:
                    s.sensorupdate({"RMAP Status" : 2}) 
                    #rmap.rmap_print("Error:Command too short:"+cmd)                                                  
            # Handles shell commands
            elif (split_cmd[0] == 'SHELL'):
                if len(split_cmd) > 1:
                    s.sensorupdate({"RMAP Status" : 3})  
                    command = cmd[6:]
                    subprocess.call(command, shell=True) 
                else: 
                    s.sensorupdate({"RMAP Status" : 2})  
                    rmap.rmap_print("Error:Command too short, expected more than 1 received "+str(len(cmd)) +":"+cmd)  
            # Handles unrecognized commands
            else:
                rmap.rmap_print("Error:Command not recognized:"+split_cmd[0]) 
                s.sensorupdate({"RMAP Status" : 2})                
        command_array = []         
        #time.sleep(.15)
        if(psm.isKeyPressed() == True): # if the GO button is pressed
            psm.screen.clearScreen()
            psm.screen.termPrintln("")
            psm.screen.termPrintln("Exiting to menu")
            #time.sleep(0.2) 
            doExit = True 

    # Keyboard interrupt exception
    except KeyboardInterrupt:
        running= False
        rmap.rmap_print("Disconnected")
        break
    # Connection disruption exception
    except (scratch.ScratchError,NameError) as e:
        errorCounter = 0
        error = 1
        while (error == 1) and (doExit == False) :
            rmap.rmap_print("Disconnected")
            time.sleep(5)
            # Reconnects to Scratch 
            try:
                s = scratch.Scratch(host=rmapcfg.host, port=rmapcfg.port)
                s.broadcast('READY')
                rmap.rmap_print("Connected")
                error = 0
            except scratch.ScratchError:
                errorCounter += 1
                rmap.rmap_print("Scratch is either not opened or remote sensor connections aren't enabled")
                if ( errorCounter > 10 ):
                    psm.screen.clearScreen()
                    psm.screen.termPrintln("Scrach connection failed")
                    time.sleep(10)
                    sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# July 2015  Henry     Initial Authoring 
# Oct. 2015  Nitin     Editing and improved functionality
# Oct. 2015  Michael   Comments and documentation

## @file PiStorms.py
#  PiStorms.py defines the main interfaces used in the PiStorms library.
#  @mainpage PiStorms Library Reference
#  @section intro_sec  Introduction
#  PiStorms library provides interfaces to use PiStorms by mindsensors.com on Raspberry Pi.
#
#  At the time of this writing, PiStorms and this library can be used with the following boards:
#
#    <b>Raspberry Pi boards:</b>\n
#    model A+\n
#    model B+\n
#    2 model B
#
#  @section more_info  More Information
#  More information about PiStorms is available at: http://www.mindsensors.com/stem-education/13-pistorms-base-kit
#
#  Online documentation of this Library Reference is available at:
#  http://www.mindsensors.com/reference/PiStorms/html/
#  (Note however, the online version may not match exactly with the library files you have installed on your computer).
#
#  @section install_sec Installation Instructions
#  To download PiStorms libary and features to your Raspberry Pi:\n
#  navigate to /home/pi\n 
#  sudo wget http://www.mindsensors.com/largefiles/PiStorms.tar.gz
#
#  Untar the file using command:\n
#  cd /home/pi\n 
#  tar -zxvf PiStorms.tar.gz
#
#  Install using following commands:\n
#  cd PiStorms\n
#  ./setup.sh
#  
#  It will take a few minutes to install the software.

from PiStormsCom import PiStormsCom
from mindsensorsUI import mindsensorsUI
import time,sys,os,ctypes,math,random

## @package PiStorms
#  This module contains classes and functions necessary for the use of PiStorms from mindsensors.com

Dev_PiStorms = 1

## PiStormsSensor: This class provides functions for configuration, reading, and writing of sensors for use with PiStorms.
#  @remark
#  There is no need to initialize this class directly. This is done automatically during the PiStorms init.
class PiStormsSensor:
    
    ## Constant to specify no color
    PS_SENSOR_COLOR_NONE = 0
    ## Constant to specify black color
    PS_SENSOR_COLOR_BLACK = 1
    ## Constant to specify blue color
    PS_SENSOR_COLOR_BLUE = 2
    ## Constant to specify green color
    PS_SENSOR_COLOR_GREEN = 3
    ## Constant to specify yellow color
    PS_SENSOR_COLOR_YELLOW = 4
    ## Constant to specify red color
    PS_SENSOR_COLOR_RED = 5
    ## Constant to specify white color
    PS_SENSOR_COLOR_WHITE = 6
    ## Constant to specify brown color
    PS_SENSOR_COLOR_BROWN = 7
    
    ### @cond 
    ## Initialize the PiStorms sensor port
    #  @param self The object pointer.
    #  @param sensor The sensor port to use.
    def __init__(self,sensor):
        self.pssensor = sensor
    ### @endcond
    
    ## Returns the EV3 Touch Sensor touch value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  touch = psm.BAS1.isTouchedEV3()
    #  if(touch == True ):
    #      # do some task
    #  @endcode      
    def isTouchedEV3(self):
        return self.pssensor.isTouchedEV3()
        
    ## Returns the EV3 Touch Sensor touch count
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  touchCount = psm.BAS1.numTouchesEV3()
    #  if(touchCount == 5):
    #      #do some task
    #  @endcode 
    def numTouchesEV3(self):
        return self.pssensor.numTouchesEV3()
    
    ## Resets the EV3 Touch Sensor touch count to zero
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAS1.resetTouchesEV3()
    #  @endcode 
    def resetTouchesEV3(self):
        self.pssensor.resetTouchesEV3()
    
    ## Returns the EV3 Infrared Sensor distance value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  dist = psm.BAS1.distanceIREV3()
    #  if(dist < 30 ):
    #      # do some task
    #  @endcode 
    def distanceIREV3(self):
        return self.pssensor.distanceIREV3()
    
    ## Returns the EV3 Infrared Sensor heading value
    #  @param self The object pointer.
    #  @param channel The channel of your remote control or other IR device.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  heading = psm.BAS1.headingIREV3(1)
    #  if(heading > 10 ):
    #      # do some task
    #  @endcode 
    def headingIREV3(self,channel):
        return self.pssensor.headingIREV3(channel)
    
    ## Returns the LEGO IR remote distance value
    #  @param self The object pointer.
    #  @param channel The channel of your remote control or other IR device.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  remDist = psm.BAS1.distanceRemoteIREV3(1)
    #  if(remDist < 50 ):
    #      # do some task
    #  @endcode 
    def distanceRemoteIREV3(self,channel):
        return self.pssensor.distanceRemoteIREV3(channel)
    
    ## Returns the left button status of the LEGO IR remote (1 = Forward, 0 = Not pressed, -1 = Backwards)
    #  @param self The object pointer.
    #  @param channel The channel of your remote control or other IR device. 
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  remLeftButton = psm.BAS1.remoteLeft(1)
    #  if(remLeftButton == 1 ):
    #      # do some task
    #  @endcode 
    def remoteLeft(self,channel):
        return self.pssensor.remoteLeft(channel)
    
    ## Returns the right button status of the LEGO IR remote (1 = Forward, 0 = Not pressed, -1 = Backwards)
    #  @param self The object pointer.
    #  @param channel The channel of your remote control or other IR device.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  remRightButton = psm.BAS1.remoteRight(1)
    #  if(remRightButton == 1 ):
    #      # do some task
    #  @endcode 
    def remoteRight(self,channel):
        return self.pssensor.remoteRight(channel)
    
    ## Returns the EV3 Ultrasonic Sensor distance value in centimeters
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  dist = psm.BAS1.distanceUSEV3()
    #  if(dist < 60 ):
    #      # do some task
    #  @endcode 
    def distanceUSEV3(self):
        return self.pssensor.distanceUSEV3cm()
    
    ## Returns the EV3 Ultrasonic Sensor distance value in inches
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  dist = psm.BAS1.distanceUSEV3in()
    #  if(dist < 100 ):
    #      # do some task
    #  @endcode 
    def distanceUSEV3in(self):
        return self.pssensor.distanceUSEV3in()
        
    ## Returns the EV3 Ultrasonic Sensor presence boolean value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  here = psm.BAS1.presenceUSEV3()
    #  if(here == True ):
    #      # do some task
    #  @endcode 
    def presenceUSEV3(self):
        return self.pssensor.presenceUSEV3()
    
    ## Returns the EV3 Gyro Sensor angle value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  angle = psm.BAS1.gyroAngleEV3()
    #  if(angle > 90 ):
    #      # do some task
    #  @endcode 
    def gyroAngleEV3(self):
        return self.pssensor.gyroAngleEV3()
    
    ## Returns the EV3 Gyro Sensor rate value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  rate = psm.BAS1.gyroRateEV3()
    #  if(rate > 20 ):
    #      # do some task
    #  @endcode 
    def gyroRateEV3(self):
        return self.pssensor.gyroRateEV3()
    
    ## Returns the EV3 Color Sensor reflected light value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  refLight = psm.BAS1.reflectedLightSensorEV3()
    #  if(refLight > 20 ):
    #      # do some task
    #  @endcode 
    def reflectedLightSensorEV3(self):
        return self.pssensor.reflectedLightSensorEV3()
    
    ## Returns the EV3 Color Sensor ambient light value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  ambLight = psm.BAS1.ambientLightSensorEV3()
    #  if(ambLight > 20 ):
    #      # do some task
    #  @endcode 
    def ambientLightSensorEV3(self):
        return self.pssensor.ambientLightSensorEV3()
    
    ## Returns the EV3 Color Sensor color value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  color = psm.BAS1.colorSensorEV3()
    #  if(color == 5 ):
    #      # do some task
    #  @endcode 
    def colorSensorEV3(self):
        return self.pssensor.colorSensorEV3()
    
    ## Returns the NXT Touch Sensor touch value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  touch = psm.BAS1.isTouchedNXT()
    #  if(touch == True):
    #      # do some task
    #  @endcode 
    def isTouchedNXT(self):
        return self.pssensor.isTouchedNXT()
    
    ## Returns the NXT Touch Sensor touch count
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  touchCount = psm.BAS1.numTouchesNXT()
    #  if(touchCount == 5 ):
    #      # do some task
    #  @endcode 
    def numTouchesNXT(self):
        return self.pssensor.numTouchesNXT()
    
    ## Resets the NXT Touch Sensor touch count to zero
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAS1.resetTouchesNXT()
    #  @endcode 
    def resetTouchesNXT(self):
        self.pssensor.resetTouchesNXT()
    
    ## Returns the mindsensors.com's SumoEyes value (0 = None, 1 = Front, 2 = Left, 3 = Right)
    #  @param self The object pointer.
    #  @param long The mode of the SumoEyes sensor. (True = long range | False = short range)
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  sumo = psm.BAS1.SumoEyes(True)
    #  if(sumo == 2 ):
    #      # do some task
    #  @endcode 
    def SumoEyes(self, long=True):
        return self.pssensor.SumoEyes(long)
    
    ## Returns the NXT light sensor value
    #  @param self The object pointer.
    #  @param active The mode of the nxt light sensor. (True = active/reflected | False = nonactive/ambient)
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  light = psm.BAS1.lightSensorNXT(True)
    #  if(light < 450 ):
    #      # do some task
    #  @endcode 
    def lightSensorNXT(self, active=True):
        return self.pssensor.lightSensorNXT(active)
    
    ## Returns the NXT color sensor color value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  color = psm.BAS1.colorSensorNXT()
    #  if(color == 5):
    #      # do some task
    #  @endcode 
    def colorSensorNXT(self):
        return self.pssensor.colorSensorNXT()
    
    ## Returns the NXT color sensor ambient light value
    #  @param self The object pointer. 
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  ambLight = psm.BAS1.colorSensorNoneNXT()
    #  if(ambLight > 20 ):
    #      # do some task
    #  @endcode 
    def colorSensorNoneNXT(self):
        return self.pssensor.colorSensorNoneNXT()
    
    ## Returns the NXT color sensor reflected red light value
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  redRefLight = psm.BAS1.colorSensorRedNXT()
    #  if(redRefLight > 20 ):
    #      # do some task
    #  @endcode 
    def colorSensorRedNXT(self):
        return self.pssensor.colorSensorRedNXT()
    
    ## Returns the NXT color sensor reflected green light value
    #  @param self The object pointer. 
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  greenRefLight = psm.BAS1.colorSensorGreenNXT()
    #  if(greenRefLight > 20 ):
    #      # do some task
    #  @endcode     
    def colorSensorGreenNXT(self):
        return self.pssensor.colorSensorGreenNXT()
    
    ## Returns the NXT color sensor reflected blue light value
    #  @param self The object pointer. 
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  blueRefLight = psm.BAS1.colorSensorBlueNXT()
    #  if(blueRefLight > 20 ):
    #      # do some task
    #  @endcode     
    def colorSensorBlueNXT(self):
        return self.pssensor.colorSensorBlueNXT()
    
    ## Returns an analog sensor value 0-1024
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  analog = psm.BAS1.analogSensor()
    #  if(analog < 600):
    #      # do some task
    #  @endcode 
    def analogSensor(self):
        return self.pssensor.analogSensor()
    
    ## Configures the sensor port for a custom I2C sensor
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAS1.activateCustomSensorI2C()
    #  @endcode 
    def activateCustomSensorI2C(self):
        self.pssensor.activateCustomSensorI2C()

## PiStormsMotor: This class provides functions for motor control.
#  @remark
#  There is no need to initialize this class directly. This is done automatically during the PiStorms init.
class PiStormsMotor():
    
    ### @cond
    # Initialize the PiStorms motor port
    #  @param self The object pointer.
    #  @param motor The motor port to use.
    def __init__(self, motor):
        self.psmotor = motor
    ### @endcond
    
    ## Returns the current encoder position of the motor
    #  @param self The object pointer.  
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  position = psm.BAM1.pos()
    #  if(position > 720):
    #      # do some task
    #  @endcode
    def pos(self):
        return self.psmotor.pos()
        
    #def resPos(self, bnk):
    #    return self.psmotor.resPos(bnk)
    
    ## Run the motor at a set speed for an unlimited duration
    #  @param self The object pointer.
    #  @param speed The speed at which to turn the motor.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.setSpeed(100)
    #  @endcode   
    def setSpeed(self,speed):
        self.psmotor.setSpeed(speed)
    
    ## Stop the motor abruptly with brake
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.brake()
    #  @endcode   
    def brake(self):
        self.psmotor.brake()
    
    ## Stop the motor smoothly with float
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.float()
    #  @endcode   
    def float(self):
        self.psmotor.float()
    
    ## Stop the motor abruptly and hold the current position
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.hold()
    #  @endcode   
    def hold(self):
        self.psmotor.hold()
    
    ## Run the motor for a specific time in seconds
    #  @param self The object pointer.
    #  @param secs The number of seconds to run the motor.
    #  @param speed The speed at which to turn the motor.
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float).
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.runSecs(5,100,True)
    #  @endcode   
    def runSecs(self,secs,speed,brakeOnCompletion = False):
        self.psmotor.runSecs(secs,speed,brakeOnCompletion)
    
    ## Check if the motor is running
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  motorState = psm.BAM1.isBusy()
    #  if(motorState == False):
    #      # do some task
    #  @endcode   
    def isBusy(self):
        return self.psmotor.isBusy()
    
    ## Wait until the motor is no longer running
    #  @param self The object pointer.
    #  @param timeout The timeout value as a factor of 10ms.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  boolMotorState = psm.BAM1.waitUntilNotBusy()
    #  if(overloadState == True):
    #      # do some task
    #  @endcode   
    def waitUntilNotBusy(self,timeout=-1):
        return self.psmotor.waitUntilNotBusy(timeout)
    
    ## Check if the motor is stalled
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  stallState = psm.BAM1.isStalled()
    #  if(stallState == True):
    #      # do some task
    #  @endcode   
    def isStalled(self):
        return self.psmotor.isStalled()
    
    ## Check if the motor is overloaded
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  overloadState = psm.BAM1.isOverloaded()
    #  if(overloadState == True):
    #      # do some task
    #  @endcode   
    def isOverloaded(self):
        return self.psmotor.isOverloaded()
    
    ## Run the motor for a specific amount of degrees
    #  @param self The object pointer.
    #  @param degs The number of degrees to run the motor.
    #  @param speed The speed at which to turn the motor.
    #  @param brakeOnCompletion Choose to brake or float the motor upon completion with True (brake) or False (float).
    #  @param holdOnCompletion Choose to hold the motor position upon completion with True (hold) or False (release).
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.runDegs(720,100,True,False)
    #  @endcode   
    def runDegs(self,degs,speed = 100,brakeOnCompletion = False, holdOnCompletion = False):
        self.psmotor.runDegs(degs,speed,brakeOnCompletion,holdOnCompletion)
    
    ## Set the motor PID parameters (PiStorms PID parameters are default for NXT/EV3 motors). Only one set of PID parameters can be set per bank and will reset to default upon new power cycle
    #  @param self The object pointer.
    #  @param Kp_tacho Proportional-gain of the encoder position of the motor.
    #  @param Ki_tacho Integral-gain of the encoder position of the motor.
    #  @param Kd_tacho Derivative-gain of the encoder position of the motor.
    #  @param Kp_speed Proportional-gain of the speed of the motor.
    #  @param Ki_speed Integral-gain of the speed of the motor.
    #  @param Kd_speed Derivative-gain of the speed of the motor.
    #  @param passcount The number of times the encoder reading should be within tolerance.
    #  @param tolerance The tolerance (in ticks) for encoder positioning .
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.BAM1.SetPerformanceParameters(6,0,0,1,0,0,10,5)
    #  @endcode   
    def SetPerformanceParameters(self, Kp_tacho, Ki_tacho, Kd_tacho, Kp_speed, Ki_speed, Kd_speed, passcount, tolerance):
        self.psmotor.SetPerformanceParameters(Kp_tacho, Ki_tacho, Kd_tacho, Kp_speed, Ki_speed, Kd_speed, passcount, tolerance)
    

## PiStorms: This class provides functions for PiStorms.
#  PiStormsSensor, PiStormsMotor, and mindsensorsUI are subclasses of PiStorms and are automatically initialized with initialization of PiStorms class.
class PiStorms:
    
    ## Initialize the PiStorms motor and sensor ports
    #  @param self The object pointer.
    #  @param name The display title that will appear at the top of the LCD touchscreen.
    #  @param rotation The rotation of the LCD touchscreen.
    #  @remark
    #  There is no need to use this function directly. To initialize the PiStorms class in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  @endcode    
    def __init__(self, name = "PiStorms", rotation = 3 ):
        
        self.screen = mindsensorsUI(name ,rotation)
        self.psc = PiStormsCom()
        self.BAS1 = PiStormsSensor(self.psc.BAS1)
        self.BAS2 = PiStormsSensor(self.psc.BAS2)
        self.BBS1 = PiStormsSensor(self.psc.BBS1)
        self.BBS2 = PiStormsSensor(self.psc.BBS2)
            
        self.BAM1 = PiStormsMotor(self.psc.BAM1)
        self.BAM2 = PiStormsMotor(self.psc.BAM2)
        self.BBM1 = PiStormsMotor(self.psc.BBM1)
        self.BBM2 = PiStormsMotor(self.psc.BBM2)
        
    def command (self, cmd, bank):
        self.psc.command(cmd, bank)
        
    #Shutdown command not yet implemented 
    # def Shutdown(self):
        # self.psc.Shutdown()
        
    ## Resets encoders for motors on BankA
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.resPosA()
    #  @endcode        
    def resPosA(self):
        self.psc.resPosA() 
        
    ## Resets encoders for motors on BankB
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.resPosA()
    #  @endcode        
    def resPosB(self):
        self.psc.resPosB()
        
    ## Returns the input battery voltage
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  volts = psm.battVoltage()
    #  if(volts > 6):
    #      # do some task
    #  @endcode    
    def battVoltage(self):
        return self.psc.battVoltage()
    
    ## Returns the PiStorms firmware version
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  fwVersion = psm.GetFirmwareVersion()
    #  print str(devID)
    #  @endcode    
    def GetFirmwareVersion(self):
        return self.psc.GetFirmwareVersion()
    
    ## Returns the PiStorms vendor name
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  venName = psm.GetVendorName()
    #  print str(devID)
    #  @endcode    
    def GetVendorName(self):
        return self.psc.GetVendorName()
    
    ## Returns the PiStorms device ID
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  devID = psm.GetDeviceId()
    #  print str(devID)
    #  @endcode    
    def GetDeviceId(self):
        return self.psc.GetDeviceId()
    
    ## Writes to the specified RGB LED
    #  @param self The object pointer.
    #  @param lednum The number to specify the LED (1 for BankA, 2 for BankB).
    #  @param red The red value to write to the specified LED (0-255).
    #  @param green The green value to write to the specified LED (0-255).
    #  @param blue The blue value to write to the specified LED (0-255).
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  psm.led(1,255,0,0)
    #  @endcode    
    def led(self,lednum,red,green,blue):
        return self.psc.led(lednum,red,green,blue)
    
    ## Check if the GO button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.isKeyPressed()
    #  if(key == True):
    #      # do some task
    #  @endcode    
    def isKeyPressed(self):
        return self.psc.isKeyPressed()

    ## Check if any Function button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.getKeyValue()
    #  if(key == 40):
    #      # (40 is F4), do some task
    #  @endcode    
    def getKeyPressValue(self):
        return self.psc.getKeyPressValue()

    ## Check if F1 Function button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.isF1Pressed()
    #  if(key == True):
    #      # F1 is pressed, do some task
    #  @endcode    
    def isF1Pressed(self):
        return (self.psc.getKeyPressValue() == 8)

    ## Check if F2 Function button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.isF2Pressed()
    #  if(key == True):
    #      # F2 is pressed, do some task
    #  @endcode    
    def isF2Pressed(self):
        return (self.psc.getKeyPressValue() == 16)

    ## Check if F3 Function button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.isF3Pressed()
    #  if(key == True):
    #      # F3 is pressed, do some task
    #  @endcode    
    def isF3Pressed(self):
        return (self.psc.getKeyPressValue() == 24)

    ## Check if F4 Function button is pressed
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  key = psm.isF4Pressed()
    #  if(key == True):
    #      # F4 is pressed, do some task
    #  @endcode    
    def isF4Pressed(self):
        return (self.psc.getKeyPressValue() == 40)
    
    ## Returns the GO button press count
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  keyCount = psm.getKeyPressCount()
    #  if(keyCount == 5):
    #      # do some task
    #  @endcode   
    def getKeyPressCount(self):
        return self.psc.getKeyPressCount()
    
    ## Resets the GO button press count
    #  @param self The object pointer.
    #  @remark
    #  To use this function in your program:
    #  @code
    #  from PiStorms import PiStorms 
    #  ...
    #  psm = PiStorms()
    #  resetKeyPressCount()
    #  @endcode   
    def resetKeyPressCount(self):
        self.psc.resetKeyPressCount()
    
    ### @cond
    ## Pings the PiStorms for reliable I2C communication
    #  @param self The object pointer.
    def ping(self):
        self.psc.ping()
    ### @endcond
    

if __name__ == '__main__':
    psm = PiStorms("PiStorms",rotation =3)
    buttonCount = 0
    psm.BBS1.resetTouchesEV3()
    print "Version = "+ str(psm.GetFirmwareVersion())
    print "Vendor = "+ str(psm.GetVendorName())
    print "Device = "+ str(psm.GetDeviceId())
    psm.screen.termPrintAt(3," Version is  "+ str(psm.GetFirmwareVersion() )[:5])
    psm.screen.termPrintAt(4," Version is  "+ str(psm.GetVendorName() ))
    psm.screen.termPrintAt(5," Version is  "+ str(psm.GetDeviceId() ))
   
    while(True):
        time.sleep(.300)
        print ' Touched at ( '+str(psm.screen.TS_X())+' , '+str(psm.screen.TS_Y())+')'
        psm.screen.termPrintAt(6,' Touched at  ('+str(psm.screen.TS_X())+' , '+str(psm.screen.TS_Y())+')')
        print " Voltage = "+str( psm.battVoltage())
        psm.screen.termPrintAt(7," Voltage = "+str( psm.battVoltage()))
        
        




#!/usr/bin/env python
#
# Copyright (c) 2016 mindsensors.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# mindsensors.com invests time and resources providing this open source code,
# please support mindsensors.com  by purchasing products from mindsensors.com!
# Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date          Author          Comments
# March 2016    Roman Bohuk     Initial Authoring

# Setup (to be present in all programs)
import os,sys,inspect,time,thread
import socket,fcntl,struct
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir)

# PiStorms and Input classes
from PiStorms import PiStorms
from PiStormsInput import PiStormsInput

# Create an instance of PiStorms class
psm = PiStorms()

# Create an instance of the PiStormsInput class
textbox = PiStormsInput(psm)

# Take user input
userInput = textbox.getInput()

# Take user input while hiding the characters (replaces with *)
userInput = textbox.getInput(hide = True)

if userInput["submitted"]:
    print "User entered: " + userInput["response"]
else:
    print "User cancelled the operation, but he entered: " + userInput["response"]

# By default, the widget has a 20px margin on the left
# Set left_padding to False so that the widget takes up the whole screen
textbox2 = PiStormsInput(psm, left_padding = False)

userInput = textbox2.getInput()

userInput = textbox2.getInput(hide = True)





#!/usr/bin/env python
#
# Copyright (c) 2016 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date      Author      Comments
# 01/01/16   Deepak     Initial development.
#
import json, time, os, sys, inspect, thread
from ws4py.client.threadedclient import WebSocketClient
from swarmclient import *
from PiStorms import PiStorms

#
# message sender 
#
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 
psm = PiStorms()
doExit = False
bmpw = 60
old_x = 110
old_y = 80

psm.screen.clearScreen()
if __name__ == '__main__':
    #
    # this callback handler handles the messages
    # received from the swarm server.
    #
    def myHandler(msg_array):
        global old_x
        global old_y
        #
        # this is where you would do something with
        # the message received from peer.
        #
        if ('type' in msg_array) and (msg_array['type'] == 'peermessage'):
            print "Peer says: ", msg_array['message']
            msg =  json.loads(str(msg_array['message']))
            if (msg['action'] == "move"):
                smiley_x = msg['x']
                smiley_y = msg['y']
                psm.screen.fillBmp(old_x, old_y, bmpw, bmpw, path = currentdir+'/'+"black-square.png")
                psm.screen.fillBmp(smiley_x, smiley_y, bmpw, bmpw, path = currentdir+'/'+"smiley.png")
                old_x = smiley_x
                old_y = smiley_y
        else:
            print "System Message: ", msg_array['message']

    psm.screen.drawAutoText("Searching Swarm neighbors ...", 15, 218, fill=(255, 255, 255), size = 18) 
    nbrs_list = find_swarm_neighbors()
    if ( len(nbrs_list) == 0 ):
        m = ["Swarm-Demo", "A swarm requires at least two PiStorms", "robots.",
                           "Get another PiStorms robot to add to", "your swarm."]
        psm.screen.askQuestion(m,["OK"])

    psm.screen.clearScreen()
    psm.screen.fillBmp(old_x, old_y, bmpw, bmpw, path = currentdir+'/'+"smiley.png")
    peers = len(nbrs_list)
    psm.screen.drawAutoText( str(peers) + " neighbor(s) found", 15, 200, fill=(255, 255, 255), size = 18) 
    psm.screen.drawAutoText("Press Go to Exit", 15, 218, fill=(255, 255, 255), size = 18) 
    #
    # register with the swarm server
    # Function parameters:
    # SwarmClient(messageHandler, <optional server>)
    #
    print "creating SwarmClient " 
    try:
        ws = SwarmClient(myHandler)
        #
        #
        if not ( ws.isRegistered ):
            print "registration failed"
            m = ["Swarm-Demo", "Swarm server registration failed."]
            psm.screen.askQuestion(m,["OK"])
            exit()

        old_tsx = 0
        old_tsy = 0
        while doExit == False:
            if ( psm.screen.isTouched() ):
                tsx = psm.screen.TS_X()
                tsy = psm.screen.TS_Y()
                tsx_delta = abs(tsx - old_tsx)
                tsy_delta = abs(tsy - old_tsy)
                # ignore small movements
                if ((tsx != 0 and tsy != 0) and (tsx_delta > 8 or tsy_delta > 8)):
                    old_tsx = tsx
                    old_tsy = tsy
                    #print "at: tsx: " + str(tsx) + " tsy: " + str(tsy)
                    # center the image where user touched.
                    image_x = psm.screen.TS_To_ImageCoords_X(tsx,tsy) - 20
                    image_y = psm.screen.TS_To_ImageCoords_Y(tsx,tsy) - 20
                    #print "touched at: x: " + str(image_x) + " y: " + str(image_y)

                    #for each nbr in the list send message of new coordinates {x,y}
                    m_array = {}
                    m_array['action'] = "move"
                    m_array['x'] = image_x
                    m_array['y'] = image_y
                    for neighbor in nbrs_list:
                        ws.SendMessageToPeer(neighbor, json.dumps(m_array))
                    psm.screen.fillBmp(old_x, old_y, bmpw, bmpw, path = currentdir+'/'+"black-square.png")
                    psm.screen.fillBmp(image_x, image_y, bmpw, bmpw, path = currentdir+'/'+"smiley.png")
                    old_x = image_x
                    old_y = image_y
                        

            if(psm.isKeyPressed() == True): # if the GO button is pressed
                psm.screen.clearScreen()
                psm.screen.termPrintln("")
                psm.screen.termPrintln("Exiting to menu")
                #time.sleep(0.2) 
                doExit = True 
            pass

    except KeyboardInterrupt:
        ws.close()




#!/usr/bin/env python
#
# Copyright (c) 2015 mindsensors.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#mindsensors.com invests time and resources providing this open source code, 
#please support mindsensors.com  by purchasing products from mindsensors.com!
#Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date        Author      Comments
#  04-12-16   Deepak      Troubleshooting program

import time
import Image
import ImageDraw
import ImageFont
import Adafruit_ILI9341 as TFT
import Adafruit_GPIO.SPI as SPI

print "Beginning PiStorms screen test"
disp = TFT.ILI9341(24, rst=25, spi=SPI.SpiDev(0,0,max_speed_hz=64000000))
disp.begin()
disp.clear()

image = disp.buffer
text = "PiStorms screen test"
fill = (255,255,255)
angle = 90
font = ImageFont.truetype("/usr/share/fonts/truetype/freefont/FreeSans.ttf", 20)
draw = ImageDraw.Draw(image)
width, height = draw.textsize(text, font=font)
textimage = Image.new('RGBA', (width, height), (0,0,0,0))
textdraw = ImageDraw.Draw(textimage)
textdraw.text((0,0), text, font=font, fill=fill)
rotated = textimage.rotate(angle, expand=1)
position = (120,100)
image.paste(rotated, position, rotated)
disp.display()
time.sleep(2)

print "Exiting PiStorms screen test"



#!/usr/bin/env python
#
# Copyright (c) 2016 mindsensors.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# mindsensors.com invests time and resources providing this open source code,
# please support mindsensors.com  by purchasing products from mindsensors.com!
# Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date          Author          Comments
# March 2016    Roman Bohuk     Initial Authoring

# Setup (to be present in all programs)
import os,sys,inspect,time,thread
import socket,fcntl,struct
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir)

from PiStorms import PiStorms
import socket

# Globals
psm = PiStorms()

# Checks if internet is available
def available():
    try:
        socket.setdefaulttimeout(5)
        socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect(("8.8.8.8", 53))
        return True
    except Exception as e: pass
    return False

psm.screen.fillRect(0, 0, 320, 240, fill = (0,0,0), display = False)
psm.screen.drawAutoText("Testing internet connection", 20, 30, fill = (255,255,255), size = 23, display = False)
psm.screen.fillRect(20, 80, 320, 240, fill = (0,0,0), display = False)
psm.screen.fillBmp(110, 110, 100, 100, path = currentdir+'/'+'load.png', display = False)
psm.screen.fillRect(0, 0, 1, 1, fill = (0,0,0), display = True)

    
test = available()
psm.screen.fillRect(0, 0, 320, 240, fill = (0,0,0), display = False)
if test:
    psm.screen.drawAutoText("You are connected", 35, 20, fill = (0,255,0), size = 25, display = False)
    psm.screen.drawAutoText("to the internet!", 35, 50, fill = (0,255,0), size = 25, display = False)
    psm.screen.fillBmp(120, 85, 80, 80, path = currentdir+'/'+'wifi_green.png', display = False)
else:
    psm.screen.drawAutoText("You are not connected", 35, 20, fill = (255,0,0), size = 25, display = False)
    psm.screen.drawAutoText("to the internet!", 35, 50, fill = (255,0,0), size = 25, display = False)
    psm.screen.fillBmp(130, 95, 60, 60, path = currentdir+'/'+'x_red.png', display = False)
    
psm.screen.drawButton(35, 170, width = 250, height = 40, text="Continue", display=False)
psm.screen.fillRect(0, 0, 1, 1, fill = (0,0,0), display = True)
exit = False
while not exit:
    cont = psm.screen.checkButton(35, 170, 250, 40)
    if cont or psm.isKeyPressed(): exit = True



#!/usr/bin/env python
#
# Copyright (c) 2016 mindsensors.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# mindsensors.com invests time and resources providing this open source code,
# please support mindsensors.com  by purchasing products from mindsensors.com!
# Learn more product option visit us @  http://www.mindsensors.com/
#
# History:
# Date          Author          Comments
# March 2016    Roman Bohuk     Initial Authoring

import os,sys,inspect,time,thread
import socket,fcntl,struct
from PiStorms import PiStorms

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir) 

class PiStormsInput:
    
    def __init__(self, screen, left_padding = True):
        self.psm = screen
        if left_padding:
            self.lm = 20
            self.w = 50
        else:
            self.lm = 1
            self.w = 53

    def update_textbox(self, txt, hide):
        # Replace with asterics if hide is set to true
        if hide:
            txt = "*" * len(txt)
        # Resize if text is too big
        sz = 22
        top = 82
        if len(txt) > 13:
            sz = 20
            top = 84
        if len(txt) > 19:
            sz = 18
            top = 86
        if len(txt) > 24:
            sz = 16
            top = 88
        # Draw on screen
        self.psm.screen.fillRect(self.lm+2, 77, 296, 45, fill = (220,220,220), display = False)
        if len(txt) > 0:
            self.psm.screen.drawAutoText(txt, self.lm+8, top, fill = (0,0,0), size = sz, display = False)
        self.psm.screen.fillRect(0, 0, 1, 1, fill = (0,0,0), display = True)
    
    # Draws the keyboard
    def redraw(self, layout, start):
        # Available symbols
        symbols = "., !?@#$%^&*()_-+=[]{}<>\\/|~`'\""
        numbers = "0123456789"
        letters = "abcdefghijklmnopqrstuvwxyz"
        # Setting the current layout
        used = ""
        if layout == "abc": used = letters
        elif layout == "ABC": used = letters.upper()
        elif layout == "?@!": used = symbols
        elif layout == "123": used = numbers
        
        # If the index is negative, circle around 
        if start < 0: start = len(used) + start
        
        # Draw buttons
        self.psm.screen.drawButton(self.lm + self.w, 195, width = self.w, height = 45, text=used[start%len(used)], display=False, align="center")
        self.psm.screen.drawButton(self.lm + self.w * 2, 195, width = self.w, height = 45, text=used[(start+1)%len(used)], display=False, align="center")
        self.psm.screen.drawButton(self.lm + self.w * 3, 195, width = self.w, height = 45, text=used[(start+2)%len(used)], display=False, align="center")
        self.psm.screen.drawButton(self.lm + self.w * 4, 195, width = self.w, height = 45, text=used[(start+3)%len(used)], display=False, align="center")
        self.psm.screen.fillRect(0, 0, 1, 1, fill = (0,0,0), display = True)
        # Return the list of current keys
        return [used[(start+i)%len(used)] for i in xrange(4)]
    
    def getInput(self, hide=False):
        self.psm.led(2,0,0,0)
        self.psm.screen.fillRect(self.lm, 0, 320, 240, fill = (0,0,0), display = False)
        #self.psm.screen.drawAutoText("Press 'GO' to submit.", 20, 47, fill = (0,255,0), size = 23, display = False)
        # Instructions
        self.psm.screen.drawButton(self.lm, 10, (320 - self.lm) / 2, 45, text="      Cancel", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w * 3, 10, (320 - self.lm) / 2, 45, text="      Submit", display=False, align="left")
        
        # Prepare textbox
        self.psm.screen.fillRect(self.lm, 75, 320 - self.lm, 49, fill = (100,100,100), display = False)
        self.psm.screen.fillRect(self.lm + 2, 77, 320 - self.lm - 4, 45, fill = (220,220,220), display = False)
        
        #Draws the control buttons
        # Top Row
        self.psm.screen.drawButton(self.lm, 150, self.w, 45, text="Shft", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w, 150, self.w, 45, text="Abc", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w * 2, 150, self.w, 45, text="?-!", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w * 3, 150, self.w, 45, text="123", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w * 4, 150, self.w, 45, text="Clr", display=False, align="left")
        self.psm.screen.drawButton(self.lm + self.w * 5, 150, self.w, 45, text="Bsp", display=False, align="left")
        # Bottom Row
        self.psm.screen.drawButton(self.lm, 195, self.w, 45, text=" <", display=False, align="center")
        self.psm.screen.drawButton(self.lm + self.w * 5, 195, self.w, 45, text=" >", display=False, align="center")

        exit = False
        usr_input = ""
        layout = "abc"
        index = 0
        upper = False
        keys = self.redraw(layout,index)
        
        toSubmit = {}
        while(not exit):
            # Top row buttons
            shft = self.psm.screen.checkButton(self.lm, 150, self.w, 45)
            abc = self.psm.screen.checkButton(self.lm + self.w, 150, self.w, 45)
            sym = self.psm.screen.checkButton(self.lm + self.w * 2, 150, self.w, 45)
            num = self.psm.screen.checkButton(self.lm + self.w * 3, 150, self.w, 45)
            clr = self.psm.screen.checkButton(self.lm + self.w * 4, 150, self.w, 45)
            bsp = self.psm.screen.checkButton(self.lm + self.w * 5, 150, self.w, 45)
            
            cancel = self.psm.screen.checkButton(self.lm, 10, (320 - self.lm) / 2, 45)
            submit = self.psm.screen.checkButton(self.lm + self.w * 3, 10, (320 - self.lm) / 2, 45)
            
            # Bottom row buttons
            prev = self.psm.screen.checkButton(self.lm, 195, self.w, 45)
            next = self.psm.screen.checkButton(self.lm + self.w * 5, 195, self.w, 45)
            letter1 = self.psm.screen.checkButton(self.lm + self.w, 195, self.w, 45)
            letter2 = self.psm.screen.checkButton(self.lm + self.w * 2, 195, self.w, 45)
            letter3 = self.psm.screen.checkButton(self.lm + self.w * 3, 195, self.w, 45)
            letter4 = self.psm.screen.checkButton(self.lm + self.w * 4, 195, self.w, 45)
            
            # Keyboard display options
            if shft: upper = not upper
            elif abc:
                layout = "abc"
                index = 0
            elif sym:
                layout = "?@!"
                index = 0
            elif num:
                layout = "123"
                index = 0
            elif prev: index -= 4
            elif next: index += 4
            
            if upper: layout = layout.upper()
            else: layout = layout.lower()
            
            if shft or abc or sym or num or prev or next:
                keys = self.redraw(layout,index)
                continue
            
            # Input buttons
            if bsp:
                if len(usr_input) == 0:
                    for i in xrange(3):
                        self.psm.led(2,255,0,0) # flash LED 1 red
                        time.sleep(0.05)
                        self.psm.led(2,0,0,0)
                        time.sleep(0.02)
                else:
                    usr_input = usr_input[:len(usr_input)-1:]
            if clr: usr_input = ""
            elif letter1: usr_input += keys[0]
            elif letter2: usr_input += keys[1]
            elif letter3: usr_input += keys[2]
            elif letter4: usr_input += keys[3]
            
            if clr or bsp or letter1 or letter2 or letter3 or letter4:
                self.update_textbox(usr_input,hide)
                continue
            
            # Exit
            if submit: 
                exit = True
                toSubmit = {"submitted":True, "response":usr_input}
            elif self.psm.isKeyPressed() or cancel:
                exit = True
                toSubmit = {"submitted":False, "response":usr_input}
                
        
        return toSubmit
